use std::error::Error;

use bevy_ecs::{schedule::Schedules, world::World};

pub enum AppExit {
    Success,
    Error(Box<dyn Error>),
}

pub type RunnerFn = Box<dyn FnOnce() -> AppExit>;

pub struct App {
    world: World,
    runner: Option<RunnerFn>,
}

impl Default for App {
    fn default() -> Self {
        let mut world = World::new();
        world.init_resource::<Schedules>();

        Self {
            world,
            runner: None,
        }
    }
}

impl App {
    pub fn world_mut(&mut self) -> &mut World {
        &mut self.world
    }

    pub fn world(&self) -> &World {
        &self.world
    }

    pub fn run(&mut self) -> Result<(), Box<dyn Error>> {
        match self.runner.take() {
            Some(runner) => match runner() {
                AppExit::Success => Ok(()),
                AppExit::Error(e) => Err(e),
            },
            None => Err(Box::new(std::io::Error::new(
                std::io::ErrorKind::Other,
                "runner is not set",
            ))),
        }
    }

    pub fn set_runner(&mut self, runner: RunnerFn) {
        self.runner = Some(runner);
    }
}
